package br.gov.mc.sei.client;

import java.net.URL;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.faces.bean.ManagedProperty;

import org.springframework.context.support.ClassPathXmlApplicationContext;

import br.gov.arquitetura.service.UniversalManager;
import br.gov.mc.cadsei.service.GenericService;
import br.gov.mc.sei.SeiPortTypeProxy;
import br.gov.mc.sei.client.base.BaseRepository;
import br.gov.mc.sei.client.enums.SimNao;
import br.gov.mc.sei.client.enums.TipoBloco;
import br.gov.mc.sei.client.util.Constants;
import br.gov.mc.sei.client.util.SeiIntegrationException;
import br.gov.mc.sei.client.util.StringUtils;
import br.gov.mc.sei.entidade.Assunto;
import br.gov.mc.sei.entidade.Documento;
import br.gov.mc.sei.entidade.Interessado;
import br.gov.mc.sei.entidade.Procedimento;
import br.gov.mc.sei.entidade.RetornoConsultaDocumento;
import br.gov.mc.sei.entidade.RetornoConsultaProcedimento;
import br.gov.mc.sei.entidade.RetornoGeracaoProcedimento;
import br.gov.mc.sei.entidade.RetornoInclusaoDocumento;
import br.gov.mc.sei.entidade.Serie;
import br.gov.mc.sei.entidade.TipoProcedimento;
import br.gov.mc.sei.entidade.Unidade;

public class SeiRepository extends BaseRepository {

	private SeiPortTypeProxy sei;
	
	@ManagedProperty(value = "#{universalManager}")
	public UniversalManager universalManager;

	
	/**
	 * Constrói uma instância do repositório para acesso aos serviços do SEI
	 * utilizando a URL definida no resource de properties {@link sei-client.properties}.
	 * @throws Exception
	 */
	public SeiRepository() throws Exception {
		try {
			ClassPathXmlApplicationContext factory = new ClassPathXmlApplicationContext(
					"applicationContext.xml");
			GenericService generic = (GenericService) factory
					.getBean("genericService");
			String url = generic.obtemValorParametro("URL_WEB_SERVICE_SEI");
			System.out.println(url); 
			
			URL parsedUrl = new URL(url);
			this.sei = new SeiPortTypeProxy(parsedUrl.toString());
			
		} catch (Exception e) {
			URL parsedUrl = new URL((String) super.getProperty(Constants.SEI_URL));
			this.sei = new SeiPortTypeProxy(parsedUrl.toString());
		}
		
	}

	/**
	 * Constrói uma instância do repositório para acesso aos serviços do SEI 
	 * utilizando a URL passada como parâmetro.
	 * @param url | Endereço do web service do SEI.
	 * @throws Exception
	 */
	public SeiRepository(String url) throws Exception {
		URL parsedUrl = new URL(url);
		this.sei = new SeiPortTypeProxy(parsedUrl.toString());
	}

	/**
	 * Incluir um novo procedimento na base de dados do SEI.
	 * @param siglaSistema | Valor informado no cadastro do sistema realizado no SEI.
	 * @param identificacaoServico | Valor informado no cadastro do serviço realizado no SEI.
	 * @param idUnidade | Identificador da unidade no SEI.
	 * @param procedimento | Dados do processo ({@link Procedimento}) a ser gerado.
	 * @param documentos | Documentos que devem ser gerados em conjunto com o processo ({@link Documento}).
	 * Se nenhum documento for gerado informar um objeto vazio. O número máximo de documentos por chamada é 
	 * limitado através do parâmetro WS_NUM_MAX_DOCS (menu Infra/Parâmetros) no SEI.
	 * @param procedimentosRelacionados | Lista com Ids de processos que devem ser relacionados 
	 * automaticamente com o novo processo.
	 * @param unidadesEnvio | Lista com Ids de unidades para envio do processo após a geração. O processo 
	 * ficará aberto na unidade geradora e nas unidades informadas neste parâmetro.
	 * @return Uma ocorrência do objecto {@link RetornoGeracaoProcedimento}
	 * @throws SeiIntegrationException
	 */
	public RetornoGeracaoProcedimento gerarProcedimento(String siglaSistema,
			String identificacaoServico, String idUnidade,
			Procedimento procedimento, List<Documento> documentos,
			List<String> procedimentosRelacionados, List<String> unidadesEnvio) throws SeiIntegrationException {

		// Realiza a validação dos parâmetros do serviço
		SeiIntegrationException exeptions = new SeiIntegrationException();
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("siglaSistema"));
		}

		if(StringUtils.isNullOrEmpty(identificacaoServico)){
			exeptions.getMessages().add(super.getRequiredMessage("identificacaoServico"));
		}
		
		if(StringUtils.isNullOrEmpty(idUnidade)){
			exeptions.getMessages().add(super.getRequiredMessage("idUnidade"));
		}else{
			try {
				Long.parseLong(idUnidade);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idUnidade"));
			}
		}

		if(procedimento == null){
			exeptions.getMessages().add(super.getRequiredMessage("procedimento"));
		}

		if(documentos == null){
			documentos = new ArrayList<Documento>();
		}

		if(procedimentosRelacionados == null){
			procedimentosRelacionados = new ArrayList<String>();
		}

		if(unidadesEnvio == null){
			unidadesEnvio = new ArrayList<String>();
		}
		
		// Lança exceções se houverem
		if(!exeptions.getMessages().isEmpty()){
			throw exeptions;
		}
		
		try {

			Documento[] docs = new Documento[documentos.size()];
			docs = documentos.toArray(docs);

			String[] procsRelacionados = new String[procedimentosRelacionados
					.size()];
			procsRelacionados = procedimentosRelacionados
					.toArray(procsRelacionados);

			String[] unidades = new String[unidadesEnvio.size()];
			unidades = unidadesEnvio.toArray(unidades);
			if(procedimento.getNivelAcesso()==null || procedimento.getNivelAcesso().equals("")){
			    procedimento.setNivelAcesso("null");
			}
			
			return this.sei.gerarProcedimento(siglaSistema,
					identificacaoServico, idUnidade, procedimento, docs,
					procsRelacionados, unidades);

		} catch (RemoteException e) {
			throw new SeiIntegrationException(e);
		}

	}

	/**
	 * Recupera o número do procedimento e link para acesso ao mesmo da base de dados do SEI.
	 * @param siglaSistema | Valor informado no cadastro do sistema realizado no SEI.
	 * @param identificacaoServico | Valor informado no cadastro do serviço realizado no SEI.
	 * @param idUnidade | Identificador da unidade no SEI.
	 * @param protocoloProcedimento | Número do processo visível para o usuário, ex: 12.1.000000077-4
	 * @return Uma ocorrência do objeto {@link RetornoConsultaProcedimento}.
	 * @throws SeiIntegrationException
	 */
	public RetornoConsultaProcedimento consultarProcedimento(
			String siglaSistema, String identificacaoServico, String idUnidade,
			String protocoloProcedimento) throws SeiIntegrationException {
		
		// Realiza a validação dos parâmetros do serviço
		SeiIntegrationException exeptions = new SeiIntegrationException();
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("siglaSistema"));
		}
		
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("identificacaoServico"));
		}
		
		if(StringUtils.isNullOrEmpty(idUnidade)){
			exeptions.getMessages().add(super.getRequiredMessage("idUnidade"));
		}else{
			try {
				Long.parseLong(idUnidade);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idUnidade"));
			}
		}
		
		if(StringUtils.isNullOrEmpty(protocoloProcedimento)){
			exeptions.getMessages().add(super.getRequiredMessage("protocoloProcedimento"));
		}
		
		// Lança exceções se houverem
		if(!exeptions.getMessages().isEmpty()){
			throw exeptions;
		}
		
		try {
			return this.sei.consultarProcedimento(siglaSistema, identificacaoServico,
					idUnidade, protocoloProcedimento);
		} catch (RemoteException e) {
			throw new SeiIntegrationException(e);
		}
		
	}

	/**
	 * Inclui um novo documento na base de dados do SEI.
	 * @param siglaSistema | Valor informado no cadastro do sistema realizado no SEI.
	 * @param identificacaoServico | Valor informado no cadastro do serviço realizado no SEI.
	 * @param idUnidade | Identificador da unidade no SEI.
	 * @param documento | Dados do documento a ser inserido ({@link Documento}).
	 * @return Uma ocorrência do objeto {@link RetornoInclusaoDocumento}.
	 * @throws SeiIntegrationException
	 */
	public RetornoInclusaoDocumento incluirDocumento(String siglaSistema,
			String identificacaoServico, String idUnidade, Documento documento) throws SeiIntegrationException {
		
		// Realiza a validação dos parâmetros do serviço
		SeiIntegrationException exeptions = new SeiIntegrationException();
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("siglaSistema"));
		}
		
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("identificacaoServico"));
		}
		
		if(StringUtils.isNullOrEmpty(idUnidade)){
			exeptions.getMessages().add(super.getRequiredMessage("idUnidade"));
		}else{
			try {
				Long.parseLong(idUnidade);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idUnidade"));
			}
		}
		
		if(documento == null){
			exeptions.getMessages().add(super.getRequiredMessage("documento"));
		}
		
		// Lança exceções se houverem
		if(!exeptions.getMessages().isEmpty()){
			throw exeptions;
		}
		
		try {
			return this.sei.incluirDocumento(siglaSistema,
					identificacaoServico, idUnidade, documento);
		} catch (RemoteException e) {
			throw new SeiIntegrationException(e);
		}
		
	}

	/**
	 * Recupera o número do documento e link para acesso ao mesmo da base de dados do SEI.
	 * @param siglaSistema | Valor informado no cadastro do sistema realizado no SEI.
	 * @param identificacaoServico | Valor informado no cadastro do serviço realizado no SEI.
	 * @param idUnidade | Identificador da unidade no SEI.
	 * @param protocoloDocumento | Número do documento visível para o usuário, ex.: 0003934.
	 * @return Uma ocorrência do objecto {@link RetornoConsultaDocumento}.
	 * @throws SeiIntegrationException
	 */
	public RetornoConsultaDocumento consultarDocumento(String siglaSistema,
			String identificacaoServico, String idUnidade,
			String protocoloDocumento) throws SeiIntegrationException {
		
		// Realiza a validação dos parâmetros do serviço
		SeiIntegrationException exeptions = new SeiIntegrationException();
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("siglaSistema"));
		}
		
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("identificacaoServico"));
		}

		if(StringUtils.isNullOrEmpty(idUnidade)){
			exeptions.getMessages().add(super.getRequiredMessage("idUnidade"));
		}else{
			try {
				Long.parseLong(idUnidade);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idUnidade"));
			}
		}

		if(StringUtils.isNullOrEmpty(protocoloDocumento)){
			exeptions.getMessages().add(super.getRequiredMessage("protocoloDocumento"));
		}
		
		// Lança exceções se houverem
		if(!exeptions.getMessages().isEmpty()){
			throw exeptions;
		}
		
		try {
			return this.sei.consultarDocumento(siglaSistema, identificacaoServico,
					idUnidade, protocoloDocumento);
		} catch (RemoteException e) {
			throw new SeiIntegrationException(e);
		}
		
	}

	/**
	 * Recupera as unidades da base de dados do SEI.
	 * @param siglaSistema | Valor informado no cadastro do sistema realizado no SEI.
	 * @param identificacaoServico | Valor informado no cadastro do serviço realizado no SEI.
	 * @param idTipoProcedimento | Opcional. Tipo do processo cadastrado no serviço.
	 * @param idSerie | Opcional. Tipo do documento cadastrado no serviço.
	 * @return Uma lista de ocorrências do objeto {@link Unidade}. As unidades serão listadas 
	 * de acordo com o acesso configurado para o serviço informado. Por
	 * exemplo, caso o serviço informado seja composto por 3 operações de geração de processo nas
	 * unidades X, Y e Z então apenas estas 3 unidades retornarão. Os parâmetros IdTipoProcedimento e
	 * IdSerie podem ser informados como filtros adicionais que serão aplicados nas operações do
	 * serviço. Desta forma a lista de retorno pode ser montada no sistema cliente apenas com valores
	 * válidos (evitando chamadas com valores não liberados para o serviço no SEI).
	 * @throws SeiIntegrationException
	 */
	public List<Unidade> listarUnidades(String siglaSistema,
			String identificacaoServico, String idTipoProcedimento,
			String idSerie) throws SeiIntegrationException {
		
		// Realiza a validação dos parâmetros do serviço
		SeiIntegrationException exeptions = new SeiIntegrationException();
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("siglaSistema"));
		}
		
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("identificacaoServico"));
		}
		
		if(!StringUtils.isNullOrEmpty(idTipoProcedimento)){
			try {
				Long.parseLong(idTipoProcedimento);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idTipoProcedimento"));
			}
		}
		
		if(!StringUtils.isNullOrEmpty(idSerie)){
			try {
				Long.parseLong(idSerie);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idSerie"));
			}
		}
		
		// Lança exceções se houverem
		if(!exeptions.getMessages().isEmpty()){
			throw exeptions;
		}		
		
		try {
			return Arrays.asList(this.sei.listarUnidades(siglaSistema,
					identificacaoServico, idTipoProcedimento, idSerie));
		} catch (RemoteException e) {
			throw new SeiIntegrationException(e);
		}
		
	}

	/**
	 * Recupera os tipos de procedimento da base de dados do SEI.
	 * @param siglaSistema | Valor informado no cadastro do sistema realizado no SEI.
	 * @param identificacaoServico | Valor informado no cadastro do serviço realizado no SEI.
	 * @param idUnidade | Opcional. Id da unidade cadastrada no serviço.
	 * @param idSerie | Opcional. Tipo do documento cadastrado no serviço.
	 * @return Uma lista de ocorrências do objeto {@link TipoProcedimento}. Os tipos de processo 
	 * serão listados de acordo com o acesso configurado para o serviço informado.
	 * Por exemplo, caso o serviço informado seja composto por 3 operações de geração de processo dos
     * tipos A, B e C então apenas estes 3 tipos retornarão. Os parâmetros IdUnidade e IdSerie podem
     * ser informados como filtros adicionais que serão aplicados nas operações do serviço. Desta forma
     * a lista de retorno pode ser montada no sistema cliente apenas com valores válidos (evitando
     * chamadas com valores não liberados para o serviço no SEI).
	 * @throws SeiIntegrationException
	 */
	public List<TipoProcedimento> listarTiposProcedimento(String siglaSistema,
			String identificacaoServico, String idUnidade, String idSerie) throws SeiIntegrationException {
		
		// Realiza a validação dos parâmetros do serviço
		SeiIntegrationException exeptions = new SeiIntegrationException();
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("siglaSistema"));
		}
		
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("identificacaoServico"));
		}
		
		if(!StringUtils.isNullOrEmpty(idUnidade)){
			try {
				Long.parseLong(idUnidade);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idUnidade"));
			}
		}
		
		if(!StringUtils.isNullOrEmpty(idSerie)){
			try {
				Long.parseLong(idSerie);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idSerie"));
			}
		}
		
		// Lança exceções se houverem
		if(!exeptions.getMessages().isEmpty()){
			throw exeptions;
		}			
		
		try {
			return Arrays.asList(this.sei.listarTiposProcedimento(siglaSistema,
					identificacaoServico, idUnidade, idSerie));
		} catch (RemoteException e) {
			throw new SeiIntegrationException(e);
		}
		
	}

	/**
	 * Recupera as séries da base de dados do SEI.
	 * @param siglaSistema | Valor informado no cadastro do sistema realizado no SEI.
	 * @param identificacaoServico | Valor informado no cadastro do serviço realizado no SEI.
	 * @param idUnidade | Opcional. Id da unidade cadastrada no serviço.
	 * @param idTipoProcedimento | Opcional. Tipo do processo cadastrado no serviço.
	 * @return Uma lista de ocorrências do objeto {@link Serie}. As séries serão listadas de 
	 * acordo com o acesso configurado para o serviço informado. Por exemplo, caso o serviço 
	 * informado seja composto por 2 operações de geração de documento dos tipos A e B então 
	 * apenas estas 2 séries retornarão. Os parâmetros IdUnidade e IdTipoProcedimento podem 
	 * ser informados como filtros adicionais que serão aplicados nas operações do serviço. 
	 * Desta forma a lista de retorno pode ser montada no sistema cliente apenas com valores 
	 * válidos (evitando chamadas com valores não liberados para o serviço no SEI).
	 * @throws SeiIntegrationException
	 */
	public List<Serie> listarSeries(String siglaSistema,
			String identificacaoServico, String idUnidade,
			String idTipoProcedimento) throws SeiIntegrationException {
		
		// Realiza a validação dos parâmetros do serviço
		SeiIntegrationException exeptions = new SeiIntegrationException();
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("siglaSistema"));
		}
		
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("identificacaoServico"));
		}
		
		if(!StringUtils.isNullOrEmpty(idUnidade)){
			try {
				Long.parseLong(idUnidade);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idUnidade"));
			}
		}
		
		if(!StringUtils.isNullOrEmpty(idTipoProcedimento)){
			try {
				Long.parseLong(idTipoProcedimento);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idTipoProcedimento"));
			}
		}
		
		// Lança exceções se houverem
		if(!exeptions.getMessages().isEmpty()){
			throw exeptions;
		}	
		
		try {
			return Arrays.asList(this.sei.listarSeries(siglaSistema,
					identificacaoServico, idUnidade, idTipoProcedimento));
		} catch (RemoteException e) {
			throw new SeiIntegrationException(e);
		}
		
	}

	/**
	 * Inclui um novo bloco na base de dados do SEI.
	 * @param siglaSistema | Valor informado no cadastro do sistema realizado no SEI.
	 * @param identificacaoServico | Valor informado no cadastro do serviço realizado no SEI.
	 * @param idUnidade | Id da unidade onde será gerado o bloco.
	 * @param tipo | Tipo do bloco definido pelo enumerador {@link TipoBloco}.
	 * @param descricao | Descrição do bloco.
	 * @param unidadesDisponibilizacao | Lista de ocorrências do objeto {@link Unidade} 
	 * (passar uma lista vazia caso o bloco não deva ser disponibilizado).
	 * @param documentos | Lista de protocolos de documentos (número visível para o usuário, ex.: 0003934)
	 * @param sinDisponibilizar | Sinalizador indicando se o bloco deve ser automaticamente disponibilizado.
	 * Valor definido pelo enumerador {@link SimNao}.
	 * @return Retorna o número do bloco gerado.
	 * @throws SeiIntegrationException
	 */
	public String gerarBloco(String siglaSistema, String identificacaoServico,
			String idUnidade, TipoBloco tipo, String descricao,
			List<String> unidadesDisponibilizacao, List<String> documentos,
			SimNao sinDisponibilizar) throws SeiIntegrationException {
		
		// Realiza a validação dos parâmetros do serviço
		SeiIntegrationException exeptions = new SeiIntegrationException();
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("siglaSistema"));
		}
		
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("identificacaoServico"));
		}
		
		if(StringUtils.isNullOrEmpty(idUnidade)){
			exeptions.getMessages().add(super.getRequiredMessage("idUnidade"));
		}else{
			try {
				Long.parseLong(idUnidade);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idUnidade"));
			}
		}
		
		if(tipo == null){
			exeptions.getMessages().add(super.getRequiredMessage("tipo"));
		}
		
		if(StringUtils.isNullOrEmpty(descricao)){
			exeptions.getMessages().add(super.getRequiredMessage("descricao"));
		}

		// Passar uma lista vazia caso o bloco não deva ser disponibilizado
		if(unidadesDisponibilizacao == null){
			unidadesDisponibilizacao = new ArrayList<String>();
		}
		
		if(StringUtils.isNullOrEmpty(documentos)){
			exeptions.getMessages().add(super.getRequiredMessage("documentos"));
		}
		
		if(sinDisponibilizar == null){
			exeptions.getMessages().add(super.getRequiredMessage("sinDisponibilizar"));
		}
		
		// Lança exceções se houverem
		if(!exeptions.getMessages().isEmpty()){
			throw exeptions;
		}
		
		try {
			String[] unidades = new String[unidadesDisponibilizacao.size()];
			unidades = unidadesDisponibilizacao.toArray(unidades);

			String[] docs = new String[documentos.size()];
			docs = documentos.toArray(docs);
			return this.sei.gerarBloco(siglaSistema, identificacaoServico,
					idUnidade, tipo.getValue(), descricao, unidades, docs,
					sinDisponibilizar.getValue());
		} catch (RemoteException e) {
			throw new SeiIntegrationException(e);
		}
		
	}

	/**
	 * Excluí o bloco informado da base de dados do SEI.
	 * @param siglaSistema | Valor informado no cadastro do sistema realizado no SEI.
	 * @param identificacaoServico | Valor informado no cadastro do serviço realizado no SEI.
	 * @param idUnidade | Id da unidade onde o bloco foi gerado.
	 * @param idBloco | Número do bloco.
	 * @return Retorna true.
	 * @throws SeiIntegrationException
	 */
	public boolean excluirBloco(String siglaSistema,
			String identificacaoServico, String idUnidade, String idBloco) throws SeiIntegrationException {
		
		// Realiza a validação dos parâmetros do serviço
		SeiIntegrationException exeptions = new SeiIntegrationException();
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("siglaSistema"));
		}
		
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("identificacaoServico"));
		}
		
		if(StringUtils.isNullOrEmpty(idUnidade)){
			exeptions.getMessages().add(super.getRequiredMessage("idUnidade"));
		}else{
			try {
				Long.parseLong(idUnidade);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idUnidade"));
			}
		}
		
		if(StringUtils.isNullOrEmpty(idBloco)){
			exeptions.getMessages().add(super.getRequiredMessage("idBloco"));
		}else{
			try {
				Long.parseLong(idBloco);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idBloco"));
			}
		}
		
		// Lança exceções se houverem
		if(!exeptions.getMessages().isEmpty()){
			throw exeptions;
		}
		
		try {
			return new Boolean(this.sei.excluirBloco(siglaSistema,
					identificacaoServico, idUnidade, idBloco)).booleanValue();
		} catch (RemoteException e) {
			throw new SeiIntegrationException(e);
		}
		
	}

	/**
	 * Disponibiliza o bloco na base de dados do SEI.
	 * @param siglaSistema | Valor informado no cadastro do sistema realizado no SEI.
	 * @param identificacaoServico | Valor informado no cadastro do serviço realizado no SEI.
	 * @param idUnidade | Id da unidade onde o bloco foi gerado.
	 * @param idBloco | Número do bloco.
	 * @return Retorna true.
	 * @throws SeiIntegrationException
	 */
	public boolean disponibilizarBloco(String siglaSistema,
			String identificacaoServico, String idUnidade, String idBloco) throws SeiIntegrationException {
		
		// Realiza a validação dos parâmetros do serviço
		SeiIntegrationException exeptions = new SeiIntegrationException();
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("siglaSistema"));
		}
		
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("identificacaoServico"));
		}
		
		if(StringUtils.isNullOrEmpty(idUnidade)){
			exeptions.getMessages().add(super.getRequiredMessage("idUnidade"));
		}else{
			try {
				Long.parseLong(idUnidade);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idUnidade"));
			}
		}
		
		if(StringUtils.isNullOrEmpty(idBloco)){
			exeptions.getMessages().add(super.getRequiredMessage("idBloco"));
		}else{
			try {
				Long.parseLong(idBloco);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idBloco"));
			}
		}
		
		// Lança exceções se houverem
		if(!exeptions.getMessages().isEmpty()){
			throw exeptions;
		}		
		
		try {
			return new Boolean(this.sei.disponibilizarBloco(siglaSistema,
					identificacaoServico, idUnidade, idBloco)).booleanValue();
		} catch (RemoteException e) {
			throw new SeiIntegrationException(e);
		}
		
	}

	/**
	 * Cancela a disponibilização do bloco na base de dados do SEI. 
	 * @param siglaSistema | Valor informado no cadastro do sistema realizado no SEI.
	 * @param identificacaoServico | Valor informado no cadastro do serviço realizado no SEI.
	 * @param idUnidade | Id da unidade onde o bloco foi gerado.
	 * @param idBloco | Número do bloco.
	 * @return Retorna true.
	 * @throws SeiIntegrationException
	 */
	public boolean cancelarDisponibilizacaoBloco(String siglaSistema,
			String identificacaoServico, String idUnidade, String idBloco) throws SeiIntegrationException {
		
		// Realiza a validação dos parâmetros do serviço
		SeiIntegrationException exeptions = new SeiIntegrationException();
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("siglaSistema"));
		}
		
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("identificacaoServico"));
		}
		
		if(StringUtils.isNullOrEmpty(idUnidade)){
			exeptions.getMessages().add(super.getRequiredMessage("idUnidade"));
		}else{
			try {
				Long.parseLong(idUnidade);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idUnidade"));
			}
		}
		
		if(StringUtils.isNullOrEmpty(idBloco)){
			exeptions.getMessages().add(super.getRequiredMessage("idBloco"));
		}else{
			try {
				Long.parseLong(idBloco);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idBloco"));
			}
		}
		
		// Lança exceções se houverem
		if(!exeptions.getMessages().isEmpty()){
			throw exeptions;
		}		
		
		try {
			return new Boolean(this.sei.cancelarDisponibilizacaoBloco(siglaSistema,
					identificacaoServico, idUnidade, idBloco)).booleanValue();
		} catch (RemoteException e) {
			throw new SeiIntegrationException(e);
		}
		
	}

	/**
	 * Inclui um novo documento dentro do bloco na base de dados do SEI.
	 * @param siglaSistema | Valor informado no cadastro do sistema realizado no SEI.
	 * @param identificacaoServico | Valor informado no cadastro do serviço realizado no SEI.
	 * @param idUnidade | Id da unidade onde o bloco foi gerado.
	 * @param idBloco | Número do bloco.
	 * @param protocoloDocumento | Número do documento visível para o usuário, ex.: 0003934.
	 * @return Retorna true.
	 * @throws SeiIntegrationException
	 */
	public boolean incluirDocumentoBloco(String siglaSistema,
			String identificacaoServico, String idUnidade, String idBloco,
			String protocoloDocumento) throws SeiIntegrationException {
		
		// Realiza a validação dos parâmetros do serviço
		SeiIntegrationException exeptions = new SeiIntegrationException();
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("siglaSistema"));
		}
		
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("identificacaoServico"));
		}
		
		if(StringUtils.isNullOrEmpty(idUnidade)){
			exeptions.getMessages().add(super.getRequiredMessage("idUnidade"));
		}else{
			try {
				Long.parseLong(idUnidade);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idUnidade"));
			}
		}
		
		if(StringUtils.isNullOrEmpty(idBloco)){
			exeptions.getMessages().add(super.getRequiredMessage("idBloco"));
		}else{
			try {
				Long.parseLong(idBloco);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idBloco"));
			}
		}
		
		if(StringUtils.isNullOrEmpty(protocoloDocumento)){
			exeptions.getMessages().add(super.getRequiredMessage("protocoloDocumento"));
		}
		
		// Lança exceções se houverem
		if(!exeptions.getMessages().isEmpty()){
			throw exeptions;
		}		
		
		try {
			return new Boolean(this.sei.incluirDocumentoBloco(siglaSistema,
					identificacaoServico, idUnidade, idBloco,
					protocoloDocumento)).booleanValue();
		} catch (RemoteException e) {
			throw new SeiIntegrationException(e);
		}
		
	}
	
	/**
	 * Retira o documento de dentro do bloco na base de dados do SEI.
	 * @param siglaSistema | Valor informado no cadastro do sistema realizado no SEI.
	 * @param identificacaoServico | Valor informado no cadastro do serviço realizado no SEI.
	 * @param idUnidade | Id da unidade onde o bloco foi gerado.
	 * @param idBloco | Número do bloco.
	 * @param protocoloDocumento | Número do documento visível para o usuário, ex.: 0003934.
	 * @return Retorna true.
	 * @throws SeiIntegrationException
	 */
	public boolean retirarDocumentoBloco(String siglaSistema,
			String identificacaoServico, String idUnidade, String idBloco,
			String protocoloDocumento) throws SeiIntegrationException {
		
		// Realiza a validação dos parâmetros do serviço
		SeiIntegrationException exeptions = new SeiIntegrationException();
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("siglaSistema"));
		}
		
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("identificacaoServico"));
		}
		
		if(StringUtils.isNullOrEmpty(idUnidade)){
			exeptions.getMessages().add(super.getRequiredMessage("idUnidade"));
		}else{
			try {
				Long.parseLong(idUnidade);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idUnidade"));
			}
		}
		
		if(StringUtils.isNullOrEmpty(idBloco)){
			exeptions.getMessages().add(super.getRequiredMessage("idBloco"));
		}else{
			try {
				Long.parseLong(idBloco);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idBloco"));
			}
		}
		
		if(StringUtils.isNullOrEmpty(protocoloDocumento)){
			exeptions.getMessages().add(super.getRequiredMessage("protocoloDocumento"));
		}
		
		// Lança exceções se houverem
		if(!exeptions.getMessages().isEmpty()){
			throw exeptions;
		}	
		
		try {
			return new Boolean(this.sei.retirarDocumentoBloco(siglaSistema,
					identificacaoServico, idUnidade, idBloco,
					protocoloDocumento)).booleanValue();
		} catch (RemoteException e) {
			throw new SeiIntegrationException(e);
		}
		
	}
	
	/**
	 * Incluir procedimento dentro do bloco na base de dados do SEI.
	 * @param siglaSistema | Valor informado no cadastro do sistema realizado no SEI.
	 * @param identificacaoServico | Valor informado no cadastro do serviço realizado no SEI.
	 * @param idUnidade | Id da unidade onde o bloco foi gerado.
	 * @param idBloco | Número do bloco.
	 * @param protocoloProcedimento | Número do processo visível para o usuário, ex: 12.1.000000077-4.
	 * @return Retorna true.
	 * @throws SeiIntegrationException
	 */
	public boolean incluirProcessoBloco(String siglaSistema,
			String identificacaoServico, String idUnidade, String idBloco,
			String protocoloProcedimento) throws SeiIntegrationException {
		
		// Realiza a validação dos parâmetros do serviço
		SeiIntegrationException exeptions = new SeiIntegrationException();
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("siglaSistema"));
		}
		
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("identificacaoServico"));
		}
		
		if(StringUtils.isNullOrEmpty(idUnidade)){
			exeptions.getMessages().add(super.getRequiredMessage("idUnidade"));
		}else{
			try {
				Long.parseLong(idUnidade);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idUnidade"));
			}
		}
		
		if(StringUtils.isNullOrEmpty(idBloco)){
			exeptions.getMessages().add(super.getRequiredMessage("idBloco"));
		}else{
			try {
				Long.parseLong(idBloco);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idBloco"));
			}
		}
		
		if(StringUtils.isNullOrEmpty(protocoloProcedimento)){
			exeptions.getMessages().add(super.getRequiredMessage("protocoloProcedimento"));
		}
		
		// Lança exceções se houverem
		if(!exeptions.getMessages().isEmpty()){
			throw exeptions;
		}
		
		try {
			return new Boolean(this.sei.incluirProcessoBloco(siglaSistema,
					identificacaoServico, idUnidade, idBloco,
					protocoloProcedimento)).booleanValue();
		} catch (RemoteException e) {
			throw new SeiIntegrationException(e);
		}
		
	}
	
	/**
	 * Retira o procedimento de dentro do bloco na base de dados do SEI.
	 * @param siglaSistema | Valor informado no cadastro do sistema realizado no SEI.
	 * @param identificacaoServico | Valor informado no cadastro do serviço realizado no SEI.
	 * @param idUnidade | Id da unidade onde o bloco foi gerado.
	 * @param idBloco | Número do bloco.
	 * @param protocoloProcedimento | Número do processo visível para o usuário, ex: 12.1.000000077-4.
	 * @return
	 * @throws SeiIntegrationException
	 */
	public boolean retirarProcessoBloco(String siglaSistema,
			String identificacaoServico, String idUnidade, String idBloco,
			String protocoloProcedimento) throws SeiIntegrationException {
		
		// Realiza a validação dos parâmetros do serviço
		SeiIntegrationException exeptions = new SeiIntegrationException();
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("siglaSistema"));
		}
		
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("identificacaoServico"));
		}
		
		if(StringUtils.isNullOrEmpty(idUnidade)){
			exeptions.getMessages().add(super.getRequiredMessage("idUnidade"));
		}else{
			try {
				Long.parseLong(idUnidade);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idUnidade"));
			}
		}
		
		if(StringUtils.isNullOrEmpty(idBloco)){
			exeptions.getMessages().add(super.getRequiredMessage("idBloco"));
		}else{
			try {
				Long.parseLong(idBloco);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idBloco"));
			}
		}
		
		if(StringUtils.isNullOrEmpty(protocoloProcedimento)){
			exeptions.getMessages().add(super.getRequiredMessage("protocoloProcedimento"));
		}
		
		// Lança exceções se houverem
		if(!exeptions.getMessages().isEmpty()){
			throw exeptions;
		}		
		
		try {
			return new Boolean(this.sei.retirarProcessoBloco(siglaSistema,
					identificacaoServico, idUnidade, idBloco,
					protocoloProcedimento)).booleanValue();
		} catch (RemoteException e) {
			throw new SeiIntegrationException(e);
		}
		
	}
	
	/**
	 * Reabre o procedimento na base de dados do SEI.
	 * @param siglaSistema | Valor informado no cadastro do sistema realizado no SEI.
	 * @param identificacaoServico | Valor informado no cadastro do serviço realizado no SEI.
	 * @param idUnidade | Id da unidade onde por onde o processo tramitou.
	 * @param protocoloProcedimento | Número do processo visível para o usuário, ex: 12.1.000000077-4.
	 * @return Retorna true.
	 * @throws SeiIntegrationException
	 */
	public boolean reabrirProcesso(String siglaSistema,
			String identificacaoServico, String idUnidade,
			String protocoloProcedimento) throws SeiIntegrationException {

		// Realiza a validação dos parâmetros do serviço
		SeiIntegrationException exeptions = new SeiIntegrationException();
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("siglaSistema"));
		}
		
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("identificacaoServico"));
		}
		
		if(StringUtils.isNullOrEmpty(idUnidade)){
			exeptions.getMessages().add(super.getRequiredMessage("idUnidade"));
		}else{
			try {
				Long.parseLong(idUnidade);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idUnidade"));
			}
		}
		
		if(StringUtils.isNullOrEmpty(protocoloProcedimento)){
			exeptions.getMessages().add(super.getRequiredMessage("protocoloProcedimento"));
		}
		
		// Lança exceções se houverem
		if(!exeptions.getMessages().isEmpty()){
			throw exeptions;
		}
		
		try {
			return new Boolean(this.sei.reabrirProcesso(siglaSistema,
					identificacaoServico, idUnidade, protocoloProcedimento))
					.booleanValue();
		} catch (RemoteException e) {
			throw new SeiIntegrationException(e);
		}
		
	}
	
	/**
	 * Concluir o procedimento na base de dados do SEI.
	 * @param siglaSistema | Valor informado no cadastro do sistema realizado no SEI.
	 * @param identificacaoServico | Valor informado no cadastro do serviço realizado no SEI.
	 * @param idUnidade | Id da unidade onde o processo está aberto.
	 * @param protocoloProcedimento | Número do processo visível para o usuário, ex: 12.1.000000077-4.
	 * @return Retorna true.
	 * @throws SeiIntegrationException
	 */
	public boolean concluirProcesso(String siglaSistema,
			String identificacaoServico, String idUnidade,
			String protocoloProcedimento) throws SeiIntegrationException {

		// Realiza a validação dos parâmetros do serviço
		SeiIntegrationException exeptions = new SeiIntegrationException();
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("siglaSistema"));
		}
		
		if(StringUtils.isNullOrEmpty(siglaSistema)){
			exeptions.getMessages().add(super.getRequiredMessage("identificacaoServico"));
		}
		
		if(StringUtils.isNullOrEmpty(idUnidade)){
			exeptions.getMessages().add(super.getRequiredMessage("idUnidade"));
		}else{
			try {
				Long.parseLong(idUnidade);
			} catch (NumberFormatException e) {
				exeptions.getMessages().add(super.getMessage(Constants.CONVERTION_MESSAGE, "idUnidade"));
			}
		}
		
		if(StringUtils.isNullOrEmpty(protocoloProcedimento)){
			exeptions.getMessages().add(super.getRequiredMessage("protocoloProcedimento"));
		}
		
		// Lança exceções se houverem
		if(!exeptions.getMessages().isEmpty()){
			throw exeptions;
		}		
		
		try {
			return new Boolean(this.sei.concluirProcesso(siglaSistema,
					identificacaoServico, idUnidade, protocoloProcedimento))
					.booleanValue();
		} catch (RemoteException e) {
			throw new SeiIntegrationException(e);
		}
		
	}
	private final static String SISTEMA = "PROTOCOLO_ELETRONICO";
	private final static String SERVICO = "INTEGRACAO";

	/**
	 * @param args
	 */
	public static void main(String[] args) {
	    try {
	    	Procedimento procedimento = new Procedimento();
	    	procedimento.setIdTipoProcedimento("100001203");
	    	Assunto assunto = new Assunto();
	    	assunto.setCodigoEstruturado("042.4");
	    	Assunto[] assuntos = new Assunto[]{assunto}; 
	    	procedimento.setAssuntos(assuntos);
	    	procedimento.setInteressados(new Interessado[]{new Interessado("1.� Subdistrito da Sede - S�o Bernardo do Campo/SP","01233033993")});
	    	procedimento.setObservacao("Cadastro teste");
	    	procedimento.setNivelAcesso("null");
	    	
	        SeiRepository sei = new SeiRepository();
	        RetornoGeracaoProcedimento processo =  sei.gerarProcedimento(SISTEMA, SERVICO, "110000834",procedimento , null, null, null);
		System.out.println(processo.getProcedimentoFormatado());
	    } catch (Exception e) {
		e.printStackTrace();
	    }
	    
	}

}












